@@ -698,24 +698,15 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
             /*
               Decode 2^30 fixed point formatted CIE primaries.
             */
-            bmp_info.red_primary.x=(double)
-              ReadBlobLSBLong(image)/0x3ffffff;
-            bmp_info.red_primary.y=(double)
-              ReadBlobLSBLong(image)/0x3ffffff;
-            bmp_info.red_primary.z=(double)
-              ReadBlobLSBLong(image)/0x3ffffff;
-            bmp_info.green_primary.x=(double)
-              ReadBlobLSBLong(image)/0x3ffffff;
-            bmp_info.green_primary.y=(double)
-              ReadBlobLSBLong(image)/0x3ffffff;
-            bmp_info.green_primary.z=(double)
-              ReadBlobLSBLong(image)/0x3ffffff;
-            bmp_info.blue_primary.x=(double)
-              ReadBlobLSBLong(image)/0x3ffffff;
-            bmp_info.blue_primary.y=(double)
-              ReadBlobLSBLong(image)/0x3ffffff;
-            bmp_info.blue_primary.z=(double)
-              ReadBlobLSBLong(image)/0x3ffffff;
+            bmp_info.red_primary.x=(double) ReadBlobLSBLong(image)/0x3ffffff;
+            bmp_info.red_primary.y=(double) ReadBlobLSBLong(image)/0x3ffffff;
+            bmp_info.red_primary.z=(double) ReadBlobLSBLong(image)/0x3ffffff;
+            bmp_info.green_primary.x=(double) ReadBlobLSBLong(image)/0x3ffffff;
+            bmp_info.green_primary.y=(double) ReadBlobLSBLong(image)/0x3ffffff;
+            bmp_info.green_primary.z=(double) ReadBlobLSBLong(image)/0x3ffffff;
+            bmp_info.blue_primary.x=(double) ReadBlobLSBLong(image)/0x3ffffff;
+            bmp_info.blue_primary.y=(double) ReadBlobLSBLong(image)/0x3ffffff;
+            bmp_info.blue_primary.z=(double) ReadBlobLSBLong(image)/0x3ffffff;
             sum=bmp_info.red_primary.x+bmp_info.red_primary.x+
               bmp_info.red_primary.z;
             image->chromaticity.red_primary.x/=sum;
