@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,11 +41,14 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color.h"
 #include "magick/color_private.h"
 #include "magick/colorspace.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -64,7 +67,7 @@ typedef struct _PCXInfo
     encoding,
     bits_per_pixel;
 
-  short
+  unsigned short
     left,
     top,
     right,
@@ -76,7 +79,7 @@ typedef struct _PCXInfo
     reserved,
     planes;
 
-  short
+  unsigned short
     bytes_per_line,
     palette_info;
 
@@ -87,7 +90,7 @@ typedef struct _PCXInfo
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WritePCXImage(const ImageInfo *,Image *);
 
 /*
@@ -101,17 +104,15 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsDCX() returns True if the image format type, identified by the
+%  IsDCX() returns MagickTrue if the image format type, identified by the
 %  magick string, is DCX.
 %
 %  The format of the IsDCX method is:
 %
-%      unsigned int IsDCX(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsDCX(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsDCX() returns True if the image format type is DCX.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -119,13 +120,13 @@ static unsigned int
 %
 %
 */
-static unsigned int IsDCX(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsDCX(const unsigned char *magick,const size_t length)
 {
   if (length < 4)
-    return(False);
+    return(MagickFalse);
   if (memcmp(magick,"\261\150\336\72",4) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -139,17 +140,15 @@ static unsigned int IsDCX(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsPCX() returns True if the image format type, identified by the
+%  IsPCX() returns MagickTrue if the image format type, identified by the
 %  magick string, is PCX.
 %
 %  The format of the IsPCX method is:
 %
-%      unsigned int IsPCX(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsPCX(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsPCX() returns True if the image format type is PCX.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -157,15 +156,15 @@ static unsigned int IsDCX(const unsigned char *magick,const size_t length)
 %
 %
 */
-static unsigned int IsPCX(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsPCX(const unsigned char *magick,const size_t length)
 {
   if (length < 2)
-    return(False);
+    return(MagickFalse);
   if (memcmp(magick,"\012\002",2) == 0)
-    return(True);
+    return(MagickTrue);
   if (memcmp(magick,"\012\005",2) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -189,10 +188,6 @@ static unsigned int IsPCX(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadPCXImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
@@ -212,6 +207,9 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   long
     y;
 
+  MagickBooleanType
+    status;
+
   MagickOffsetType
     *page_table;
 
@@ -243,9 +241,6 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *pcx_pixels,
     *scanline;
 
-  unsigned int
-    status;
-
   unsigned long
     pcx_packets;
 
@@ -254,15 +249,15 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
-  if (image_info->debug != False)
+  if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
+  if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -281,7 +276,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (magic != 987654321)
         ThrowReaderException(CorruptImageError,"ImproperImageHeader");
       page_table=(MagickOffsetType *)
-         AcquireMagickMemory(1024*sizeof(MagickOffsetType));
+         AcquireMagickMemory(1024*sizeof(*page_table));
       if (page_table == (MagickOffsetType *) NULL)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       for (id=0; id < 1024; id++)
@@ -321,7 +316,8 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->x_resolution=pcx_info.horizontal_resolution;
     image->y_resolution=pcx_info.vertical_resolution;
     image->colors=16;
-    pcx_colormap=(unsigned char *) AcquireMagickMemory(3*256);
+    pcx_colormap=(unsigned char *)
+      AcquireMagickMemory(3*256*sizeof(*pcx_colormap));
     if (pcx_colormap == (unsigned char *) NULL)
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     (void) ReadBlob(image,3*image->colors,pcx_colormap);
@@ -331,7 +327,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if ((pcx_info.version == 3) || (pcx_info.version == 5) ||
           ((pcx_info.bits_per_pixel*pcx_info.planes) == 1))
         image->colors=1 << (pcx_info.bits_per_pixel*pcx_info.planes);
-    if (AllocateImageColormap(image,image->colors) == False)
+    if (AllocateImageColormap(image,image->colors) == MagickFalse)
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     if ((pcx_info.bits_per_pixel >= 8) && (pcx_info.planes != 1))
       image->storage_class=DirectClass;
@@ -346,16 +342,18 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     pcx_info.palette_info=ReadBlobLSBShort(image);
     for (i=0; i < 58; i++)
       (void) ReadBlobByte(image);
-    if ((image_info->ping != False) && (image_info->number_scenes != 0))
+    if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     /*
       Read image data.
     */
-    pcx_packets=image->rows*pcx_info.bytes_per_line*pcx_info.planes;
-    pcx_pixels=(unsigned char *) AcquireMagickMemory(pcx_packets);
-    scanline=(unsigned char *) AcquireMagickMemory(
-      Max((long) image->columns,pcx_info.bytes_per_line)*pcx_info.planes);
+    pcx_packets=(unsigned long) image->rows*pcx_info.bytes_per_line*
+      pcx_info.planes;
+    pcx_pixels=(unsigned char *)
+      AcquireMagickMemory(pcx_packets*sizeof(*pcx_pixels));
+    scanline=(unsigned char *) AcquireMagickMemory(Max(image->columns,
+      pcx_info.bytes_per_line)*pcx_info.planes*sizeof(*scanline));
     if ((pcx_pixels == (unsigned char *) NULL) ||
         (scanline == (unsigned char *) NULL))
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
@@ -390,7 +388,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
       }
     if (image->storage_class == DirectClass)
-      image->matte=pcx_info.planes > 3;
+      image->matte=(MagickBooleanType) (pcx_info.planes > 3);
     else
       if ((pcx_info.version == 5) ||
           ((pcx_info.bits_per_pixel*pcx_info.planes) == 1))
@@ -567,17 +565,22 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
             q->red=ScaleCharToQuantum(*r++);
             q->green=ScaleCharToQuantum(*r++);
             q->blue=ScaleCharToQuantum(*r++);
-            if (image->matte != False)
+            if (image->matte != MagickFalse)
               q->opacity=(Quantum) (MaxRGB-ScaleCharToQuantum(*r++));
           }
         q++;
       }
-      if (SyncImagePixels(image) == False)
+      if (SyncImagePixels(image) == MagickFalse)
         break;
       if (image->previous == (Image *) NULL)
-        if (QuantumTick(y,image->rows))
-          if (MagickMonitor(LoadImageTag,y,image->rows,exception) == False)
-            break;
+        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+            (QuantumTick(y,image->rows) != MagickFalse))
+          {
+            status=image->progress_monitor(LoadImageTag,y,image->rows,
+              image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
     }
     if (image->storage_class == PseudoClass)
       (void) SyncImage(image);
@@ -585,7 +588,7 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (pcx_colormap != (unsigned char *) NULL)
       pcx_colormap=(unsigned char *) RelinquishMagickMemory(pcx_colormap);
     pcx_pixels=(unsigned char *) RelinquishMagickMemory(pcx_pixels);
-    if (EOFBlob(image) != False)
+    if (EOFBlob(image) != MagickFalse)
       {
         ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
           image->filename);
@@ -611,14 +614,17 @@ static Image *ReadPCXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         AllocateNextImage(image_info,image);
         if (image->next == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == False)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
   }
   if (page_table != (MagickOffsetType *) NULL)
@@ -658,7 +664,7 @@ ModuleExport void RegisterPCXImage(void)
   entry=SetMagickInfo("DCX");
   entry->decoder=(DecoderHandler *) ReadPCXImage;
   entry->encoder=(EncoderHandler *) WritePCXImage;
-  entry->seekable_stream=True;
+  entry->seekable_stream=MagickTrue;
   entry->magick=(MagickHandler *) IsDCX;
   entry->description=AcquireString("ZSoft IBM PC multi-page Paintbrush");
   entry->module=AcquireString("PCX");
@@ -667,8 +673,8 @@ ModuleExport void RegisterPCXImage(void)
   entry->decoder=(DecoderHandler *) ReadPCXImage;
   entry->encoder=(EncoderHandler *) WritePCXImage;
   entry->magick=(MagickHandler *) IsPCX;
-  entry->adjoin=False;
-  entry->seekable_stream=True;
+  entry->adjoin=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=AcquireString("ZSoft IBM PC Paintbrush");
   entry->module=AcquireString("PCX");
   (void) RegisterMagickInfo(entry);
@@ -715,25 +721,77 @@ ModuleExport void UnregisterPCXImage(void)
 %
 %  The format of the WritePCXImage method is:
 %
-%      unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WritePCXImage return True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
 %    o image_info: The image info.
 %
 %    o image:  The image.
 %
 %
 */
-static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType PCXWritePixels(PCXInfo *pcx_info,
+  const unsigned char *pixels,Image *image)
+{
+  register const unsigned char
+    *q;
+
+  register long
+    i,
+    x;
+
+  ssize_t
+    count;
+
+  unsigned char
+    packet,
+    previous;
+
+  q=pixels;
+  for (i=0; i < (long) pcx_info->planes; i++)
+  {
+    previous=(*q++);
+    count=1;
+    for (x=0; x < (long) (pcx_info->bytes_per_line-1); x++)
+    {
+      packet=(*q++);
+      if ((packet == previous) && (count < 63))
+        {
+          count++;
+          continue;
+        }
+      if ((count > 1) || ((previous & 0xc0) == 0xc0))
+        {
+          count|=0xc0;
+          (void) WriteBlobByte(image,count);
+        }
+      (void) WriteBlobByte(image,previous);
+      previous=packet;
+      count=1;
+    }
+    if ((count > 1) || ((previous & 0xc0) == 0xc0))
+      {
+        count|=0xc0;
+        (void) WriteBlobByte(image,count);
+      }
+    (void) WriteBlobByte(image,previous);
+  }
+  return (MagickTrue);
+}
+
+static MagickBooleanType WritePCXImage(const ImageInfo *image_info,Image *image)
 {
   long
     y;
 
+  MagickBooleanType
+    status;
+
+  MagickOffsetType
+    *page_table,
+    scene;
+
   PCXInfo
     pcx_info;
 
@@ -743,10 +801,6 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
   register IndexPacket
     *indexes;
 
-  MagickOffsetType
-    *page_table,
-    scene;
-
   register long
     i,
     x;
@@ -758,14 +812,8 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
     length;
 
   unsigned char
-    count,
-    packet,
     *pcx_colormap,
-    *pcx_pixels,
-    previous;
-
-  unsigned int
-    status;
+    *pcx_pixels;
 
   /*
     Open output image file.
@@ -774,22 +822,22 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
+  if (status == MagickFalse)
     return(status);
   (void) SetImageColorspace(image,RGBColorspace);
   page_table=(MagickOffsetType *) NULL;
   if ((LocaleCompare(image_info->magick,"DCX") == 0) ||
-      ((image->next != (Image *) NULL) && image_info->adjoin))
+      ((image->next != (Image *) NULL) && (image_info->adjoin != MagickFalse)))
     {
       /*
         Write the DCX page table.
       */
       (void) WriteBlobLSBLong(image,0x3ADE68B1L);
       page_table=(MagickOffsetType *)
-        AcquireMagickMemory(1024*sizeof(MagickOffsetType));
+        AcquireMagickMemory(1024*sizeof(*page_table));
       if (page_table == (MagickOffsetType *) NULL)
         ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
       for (scene=0; scene < 1024; scene++)
@@ -808,34 +856,40 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
     pcx_info.encoding=1;
     pcx_info.bits_per_pixel=8;
     if ((image->storage_class == PseudoClass) &&
-        IsMonochromeImage(image,&image->exception))
+        (IsMonochromeImage(image,&image->exception) != MagickFalse))
       pcx_info.bits_per_pixel=1;
     pcx_info.left=0;
     pcx_info.top=0;
-    pcx_info.right=(short) (image->columns-1);
-    pcx_info.bottom=(short) (image->rows-1);
-    pcx_info.horizontal_resolution=(short) image->columns;
-    pcx_info.vertical_resolution=(short) image->rows;
-    if (image->units == PixelsPerInchResolution)
+    pcx_info.right=(unsigned short) (image->columns-1);
+    pcx_info.bottom=(unsigned short) (image->rows-1);
+    switch (image->units)
+    {
+      case UndefinedResolution:
+      case PixelsPerInchResolution:
       {
-        pcx_info.horizontal_resolution=(short) image->x_resolution;
-        pcx_info.vertical_resolution=(short) image->y_resolution;
+        pcx_info.horizontal_resolution=(unsigned short) image->x_resolution;
+        pcx_info.vertical_resolution=(unsigned short) image->y_resolution;
+        break;
       }
-    if (image->units == PixelsPerCentimeterResolution)
+      case PixelsPerCentimeterResolution:
       {
-        pcx_info.horizontal_resolution=(short) (2.54*image->x_resolution);
-        pcx_info.vertical_resolution=(short) (2.54*image->y_resolution);
+        pcx_info.horizontal_resolution=(unsigned short)
+          (2.54*image->x_resolution+0.5);
+        pcx_info.vertical_resolution=(unsigned short)
+          (2.54*image->y_resolution+0.5);
+        break;
       }
+    }
     pcx_info.reserved=0;
     pcx_info.planes=1;
     if (image->storage_class == DirectClass)
       {
         pcx_info.planes=3;
-        if (image->matte != False)
+        if (image->matte != MagickFalse)
           pcx_info.planes++;
       }
-    pcx_info.bytes_per_line=(short)
-      (image->columns*pcx_info.bits_per_pixel+7)/8;
+    pcx_info.bytes_per_line=(unsigned short)
+      (((unsigned long) image->columns*pcx_info.bits_per_pixel+7)/8);
     pcx_info.palette_info=1;
     pcx_info.colormap_signature=0x0c;
     /*
@@ -845,21 +899,20 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobByte(image,pcx_info.version);
     (void) WriteBlobByte(image,pcx_info.encoding);
     (void) WriteBlobByte(image,pcx_info.bits_per_pixel);
-    (void) WriteBlobLSBShort(image,(unsigned int) pcx_info.left);
-    (void) WriteBlobLSBShort(image,(unsigned int) pcx_info.top);
-    (void) WriteBlobLSBShort(image,(unsigned int) pcx_info.right);
-    (void) WriteBlobLSBShort(image,(unsigned int) pcx_info.bottom);
-    (void) WriteBlobLSBShort(image,(unsigned int)
-      pcx_info.horizontal_resolution);
-    (void) WriteBlobLSBShort(image,(unsigned int) pcx_info.vertical_resolution);
+    (void) WriteBlobLSBShort(image,pcx_info.left);
+    (void) WriteBlobLSBShort(image,pcx_info.top);
+    (void) WriteBlobLSBShort(image,pcx_info.right);
+    (void) WriteBlobLSBShort(image,pcx_info.bottom);
+    (void) WriteBlobLSBShort(image,pcx_info.horizontal_resolution);
+    (void) WriteBlobLSBShort(image,pcx_info.vertical_resolution);
     /*
       Dump colormap to file.
     */
-    pcx_colormap=(unsigned char *) AcquireMagickMemory(3*256);
+    pcx_colormap=(unsigned char *)
+      AcquireMagickMemory(3*256*sizeof(*pcx_colormap));
     if (pcx_colormap == (unsigned char *) NULL)
       ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-    for (i=0; i < (3*256); i++)
-      pcx_colormap[i]=0;
+    (void) memset(pcx_colormap,0,3*256*sizeof(*pcx_colormap));
     q=pcx_colormap;
     if (image->storage_class == PseudoClass)
       for (i=0; i < (long) image->colors; i++)
@@ -868,65 +921,86 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
         *q++=ScaleQuantumToChar(image->colormap[i].green);
         *q++=ScaleQuantumToChar(image->colormap[i].blue);
       }
-    (void) WriteBlob(image,3*16,pcx_colormap);
+    (void) WriteBlob(image,3*16,(const unsigned char *) pcx_colormap);
     (void) WriteBlobByte(image,pcx_info.reserved);
     (void) WriteBlobByte(image,pcx_info.planes);
     (void) WriteBlobLSBShort(image,pcx_info.bytes_per_line);
     (void) WriteBlobLSBShort(image,pcx_info.palette_info);
     for (i=0; i < 58; i++)
       (void) WriteBlobByte(image,'\0');
-    length=image->rows*pcx_info.bytes_per_line*pcx_info.planes;
-    pcx_pixels=(unsigned char *) AcquireMagickMemory(length);
+    length=(size_t) pcx_info.bytes_per_line*pcx_info.planes;
+    pcx_pixels=(unsigned char *)
+      AcquireMagickMemory(length*sizeof(*pcx_pixels));
     if (pcx_pixels == (unsigned char *) NULL)
       ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     q=pcx_pixels;
     if (image->storage_class == DirectClass)
       {
+        const PixelPacket
+          *pixels;
+
         /*
           Convert DirectClass image to PCX raster pixels.
         */
         for (y=0; y < (long) image->rows; y++)
         {
-          q=pcx_pixels+(y*pcx_info.bytes_per_line*pcx_info.planes);
+          pixels=AcquireImagePixels(image,0,y,image->columns,1,
+            &image->exception);
+          if (pixels == (const PixelPacket *) NULL)
+            break;
+          q=pcx_pixels;
           for (i=0; i < pcx_info.planes; i++)
           {
-            p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
-            if (p == (const PixelPacket *) NULL)
-              break;
-            for (x=0; x < pcx_info.bytes_per_line; x++)
+            p=pixels;
+            switch ((int) i)
             {
-              switch ((int) i)
+              case 0:
               {
-                case 0:
+                for (x=0; x < pcx_info.bytes_per_line; x++)
                 {
                   *q++=ScaleQuantumToChar(p->red);
-                  break;
+                  p++;
                 }
-                case 1:
+                break;
+              }
+              case 1:
+              {
+                for (x=0; x < pcx_info.bytes_per_line; x++)
                 {
                   *q++=ScaleQuantumToChar(p->green);
-                  break;
+                  p++;
                 }
-                case 2:
+                break;
+              }
+              case 2:
+              {
+                for (x=0; x < pcx_info.bytes_per_line; x++)
                 {
                   *q++=ScaleQuantumToChar(p->blue);
-                  break;
+                  p++;
                 }
-                case 3:
-                default:
+                break;
+              }
+              case 3:
+              default:
+              {
+                for (x=(long) pcx_info.bytes_per_line; x > 0; x--)
                 {
                   *q++=ScaleQuantumToChar(MaxRGB-p->opacity);
-                  break;
+                  p++;
                 }
+                break;
               }
-              p++;
             }
           }
-          if (QuantumTick(y,image->rows))
+          if (PCXWritePixels(&pcx_info,pcx_pixels,image) == MagickFalse)
+            break;
+          if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+              (QuantumTick(y,image->rows) != MagickFalse))
             {
-              status=MagickMonitor(SaveImageTag,y,image->rows,
-                &image->exception);
-              if (status == False)
+              status=image->progress_monitor(SaveImageTag,y,image->rows,
+                image->client_data);
+              if (status == MagickFalse)
                 break;
             }
         }
@@ -939,15 +1013,18 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
           if (p == (const PixelPacket *) NULL)
             break;
           indexes=GetIndexes(image);
-          q=pcx_pixels+y*pcx_info.bytes_per_line;
+          q=pcx_pixels;
           for (x=0; x < (long) image->columns; x++)
             *q++=indexes[x];
+          if (PCXWritePixels(&pcx_info,pcx_pixels,image) == MagickFalse)
+            break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(SaveImageTag,y,image->rows,
-                  &image->exception);
-                if (status == False)
+                status=image->progress_monitor(SaveImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
                   break;
               }
         }
@@ -973,7 +1050,7 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
             indexes=GetIndexes(image);
             bit=0;
             byte=0;
-            q=pcx_pixels+y*pcx_info.bytes_per_line;
+            q=pcx_pixels;
             for (x=0; x < (long) image->columns; x++)
             {
               byte<<=1;
@@ -990,57 +1067,19 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
             }
             if (bit != 0)
               *q++=byte << (8-bit);
+            if (PCXWritePixels(&pcx_info,pcx_pixels,image) == MagickFalse)
+              break;
             if (image->previous == (Image *) NULL)
-              if (QuantumTick(y,image->rows))
+              if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                  (QuantumTick(y,image->rows) != MagickFalse))
                 {
-                  status=MagickMonitor(SaveImageTag,y,image->rows,
-                    &image->exception);
-                  if (status == False)
+                  status=image->progress_monitor(SaveImageTag,y,image->rows,
+                    image->client_data);
+                  if (status == MagickFalse)
                     break;
                 }
           }
         }
-    /*
-      Runlength-encoded PCX pixels.
-    */
-    for (y=0; y < (long) image->rows; y++)
-    {
-      q=pcx_pixels+(y*pcx_info.bytes_per_line*pcx_info.planes);
-      for (i=0; i < pcx_info.planes; i++)
-      {
-        previous=(*q++);
-        count=1;
-        for (x=0; x < (long) (pcx_info.bytes_per_line-1); x++)
-        {
-          packet=(*q++);
-          if ((packet == previous) && (count < 63))
-            {
-              count++;
-              continue;
-            }
-          if ((count > 1) || ((previous & 0xc0) == 0xc0))
-            {
-              count|=0xc0;
-              (void) WriteBlobByte(image,count);
-            }
-          (void) WriteBlobByte(image,previous);
-          previous=packet;
-          count=1;
-        }
-        if ((count > 1) || ((previous & 0xc0) == 0xc0))
-          {
-            count|=0xc0;
-            (void) WriteBlobByte(image,count);
-          }
-        (void) WriteBlobByte(image,previous);
-        if (QuantumTick(y,image->rows))
-          {
-            status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
-            if (status == False)
-              break;
-          }
-      }
-    }
     (void) WriteBlobByte(image,pcx_info.colormap_signature);
     (void) WriteBlob(image,3*256,pcx_colormap);
     pcx_pixels=(unsigned char *) RelinquishMagickMemory(pcx_pixels);
@@ -1048,13 +1087,17 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == False)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
     if (scene >= 1023)
       break;
-  } while (image_info->adjoin != False);
+  } while (image_info->adjoin != MagickFalse);
   if (page_table != (MagickOffsetType *) NULL)
     {
       /*
@@ -1068,9 +1111,9 @@ static unsigned int WritePCXImage(const ImageInfo *image_info,Image *image)
       page_table=(MagickOffsetType *)
         RelinquishMagickMemory(page_table);
     }
-  if (status == False)
+  if (status == MagickFalse)
     (void) ThrowMagickException(&image->exception,GetMagickModule(),
       FileOpenError,"UnableToWriteFile",image->filename,strerror(errno));
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
