@@ -54,6 +54,7 @@
 #include "magick/log.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
+#include "magick/pixel.h"
 #include "magick/profile.h"
 #include "magick/static.h"
 #include "magick/string_.h"
@@ -269,8 +270,8 @@ static MagickBooleanType DecodeImage(Image *image,const long channel)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsPSD()() returns MagickTrue if the image format type, identified by the magick
-%  string, is PSD.
+%  IsPSD()() returns MagickTrue if the image format type, identified by the
+%  magick string, is PSD.
 %
 %  The format of the IsPSD method is:
 %
@@ -365,9 +366,10 @@ static CompositeOperator PSDBlendModeToCompositeOperator(const char *mode)
 
 static char *CompositeOperatorToPSDBlendMode(CompositeOperator inOp)
 {
-  char*  outMode = (char *) "norm";
+  char
+    *outMode = (char *) "norm";
 
-  switch ( inOp )
+  switch (inOp)
   {
     case OverCompositeOp:    outMode = (char *) "norm";  break;
     case MultiplyCompositeOp:  outMode = (char *) "mul ";  break;
@@ -385,7 +387,6 @@ static char *CompositeOperatorToPSDBlendMode(CompositeOperator inOp)
     default:
       outMode = (char *) "norm";
 /*
-
   if (LocaleNCompare(mode,"hLit",4) == 0)
     return(OverCompositeOp);
   if (LocaleNCompare(mode,"sLit",4) == 0)
@@ -396,11 +397,9 @@ static char *CompositeOperatorToPSDBlendMode(CompositeOperator inOp)
     return(OverCompositeOp);
   if (LocaleNCompare(mode,"idiv",4) == 0)
     return(OverCompositeOp);
-
 */
   }
-
-  return outMode;
+  return(outMode);
 }
 
 typedef enum
@@ -511,6 +510,8 @@ static MagickBooleanType ParseImageResourceBlocks(Image *image,
 
 static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
+#define MaxPSDChannels  24
+
   typedef struct _ChannelInfo
   {
     short int
@@ -530,7 +531,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       channels;
 
     ChannelInfo
-      channel_info[24];
+      channel_info[MaxPSDChannels];
 
     char
       blendkey[4];
@@ -647,10 +648,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
-      if (image->debug != MagickFalse)
-        (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  OpenBlob failed");
-      ThrowFileException(&image->exception,FileOpenError,"UnableToOpenFile",
-        image->filename);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -659,24 +657,21 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   count=ReadBlob(image,4,(unsigned char *) psd_info.signature);
   psd_info.version=ReadBlobMSBShort(image);
   if ((count == 0) || (LocaleNCompare(psd_info.signature,"8BPS",4) != 0) ||
-      (psd_info.version != 1)) {
-      if (image->debug != MagickFalse)
-        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-          "  File signature was %.4s instead of '8BPS'",psd_info.signature);
+      (psd_info.version != 1))
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-  }
   (void) ReadBlob(image,6,psd_info.reserved);
   psd_info.channels=ReadBlobMSBShort(image);
+  if (psd_info.channels > MaxPSDChannels)
+    ThrowReaderException(CorruptImageError,"MaximumChannelsExceeded");
   psd_info.rows=ReadBlobMSBLong(image);
   psd_info.columns=ReadBlobMSBLong(image);
   psd_info.depth=ReadBlobMSBShort(image);
   psd_info.mode=ReadBlobMSBShort(image);
-
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(CoderEvent,GetMagickModule(),
       "  Image is %ld x %ld with channels=%d, depth=%d, mode=%s",
-      psd_info.columns, psd_info.rows, psd_info.channels,
-      psd_info.depth, ModeToString((PSDImageType) psd_info.mode));
+      psd_info.columns,psd_info.rows,psd_info.channels,psd_info.depth,
+      ModeToString((PSDImageType) psd_info.mode));
   /*
     Initialize image.
   */
@@ -692,21 +687,16 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image->colorspace=CMYKColorspace;
       image->matte=psd_info.channels >= 5 ? MagickTrue : MagickFalse;
     }
-  if ((psd_info.mode == BitmapMode) ||
-      (psd_info.mode == GrayscaleMode) ||
+  if ((psd_info.mode == BitmapMode) || (psd_info.mode == GrayscaleMode) ||
       (psd_info.mode == DuotoneMode))
     {
-      if (AllocateImageColormap(image,256) == MagickFalse) {
+      if (AllocateImageColormap(image,256) == MagickFalse)
+        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+      image->matte=psd_info.channels >= 2 ? MagickTrue : MagickFalse;
       if (image->debug != MagickFalse)
         (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-          "  allocation of ImageColorMap failed");
-        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+          "  ImageColorMap allocated");
     }
-    image->matte=psd_info.channels >= 2 ? MagickTrue : MagickFalse;
-    if (image->debug != MagickFalse)
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(), "  ImageColorMap allocated");
-    }
-
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(CoderEvent,GetMagickModule(),
       image->matte ? "  image has matte" : "  image has no matte");
@@ -716,20 +706,17 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   length=ReadBlobMSBLong(image);
   if (length != 0)
     {
-    if (image->debug != MagickFalse)
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  reading colormap");
-
+      if (image->debug != MagickFalse)
+        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+          "  reading colormap");
       if (psd_info.mode == DuotoneMode)
         {
           /*
             Duotone image data;  the format of this data is undocumented.
           */
           data=(unsigned char *) AcquireMagickMemory(length);
-          if (data == (unsigned char *) NULL) {
-        if (image->debug != MagickFalse)
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  allocation of ImageColorMap failed");
+          if (data == (unsigned char *) NULL)
             ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-      }
           (void) ReadBlob(image,length,data);
           data=(unsigned char *) RelinquishMagickMemory(data);
         }
@@ -738,19 +725,16 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
           /*
             Read PSD raster colormap.
           */
-          if (AllocateImageColormap(image,length/3) == MagickFalse) {
-        if (image->debug != MagickFalse)
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  allocation of ImageColorMap failed");
+          if (AllocateImageColormap(image,length/3) == MagickFalse)
             ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-      }
-        for (i=0; i < (long) image->colors; i++)
-          image->colormap[i].red=ScaleCharToQuantum(ReadBlobByte(image));
-        for (i=0; i < (long) image->colors; i++)
-          image->colormap[i].green=ScaleCharToQuantum(ReadBlobByte(image));
-        for (i=0; i < (long) image->colors; i++)
-          image->colormap[i].blue=ScaleCharToQuantum(ReadBlobByte(image));
-        image->matte=psd_info.channels >= 2 ? MagickTrue : MagickFalse;
-      }
+          for (i=0; i < (long) image->colors; i++)
+            image->colormap[i].red=ScaleCharToQuantum(ReadBlobByte(image));
+          for (i=0; i < (long) image->colors; i++)
+            image->colormap[i].green=ScaleCharToQuantum(ReadBlobByte(image));
+          for (i=0; i < (long) image->colors; i++)
+            image->colormap[i].blue=ScaleCharToQuantum(ReadBlobByte(image));
+          image->matte=psd_info.channels >= 2 ? MagickTrue : MagickFalse;
+        }
     }
   length=ReadBlobMSBLong(image);
   if (length != 0)
@@ -763,7 +747,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       */
       if (image->debug != MagickFalse)
         (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-          "  reading image resource blocks - %ld bytes", (long) length);
+          "  reading image resource blocks - %ld bytes",(long) length);
       blocks=(unsigned char *) AcquireMagickMemory(length);
       if (blocks == (unsigned char *) NULL)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
@@ -783,13 +767,8 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (image_info->ping != MagickFalse)
     {
       CloseBlob(image);
-
-      if (image->debug != MagickFalse)
-        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-          "  pinging of image complete");
       return(GetFirstImageInList(image));
     }
-
   /*
     Layer and mask block.
   */
@@ -811,7 +790,13 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
   else
     {
-      if (length != 0)
+      if (length == 0)
+        {
+          if (image->debug != MagickFalse)
+            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+              "  image has no layers");
+        }
+      else
         {
           offset=TellBlob(image);
           size=ReadBlobMSBLong(image);
@@ -832,65 +817,75 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
             }
           layer_info=(LayerInfo *)
             AcquireMagickMemory((size_t) number_layers*sizeof(*layer_info));
-          if (layer_info == (LayerInfo *) NULL) {
-          if (image->debug != MagickFalse)
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-              "  allocation of LayerInfo failed");
-            ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-        }
-        (void) ResetMagickMemory(layer_info,0,(size_t)
-          number_layers*sizeof(*layer_info));
-        for (i=0; i < number_layers; i++)
-        {
-          if (image->debug != MagickFalse)
-              (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  reading layer #%ld", i+1);
+          if (layer_info == (LayerInfo *) NULL)
+            {
+              if (image->debug != MagickFalse)
+                (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                  "  allocation of LayerInfo failed");
+              ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+            }
+          (void) ResetMagickMemory(layer_info,0,(size_t)
+            number_layers*sizeof(*layer_info));
+          for (i=0; i < number_layers; i++)
+          {
+            if (image->debug != MagickFalse)
+              (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                "  reading layer #%ld",i+1);
             layer_info[i].page.y=(long) ReadBlobMSBLong(image);
             layer_info[i].page.x=(long) ReadBlobMSBLong(image);
-            layer_info[i].page.height=(ReadBlobMSBLong(image)-layer_info[i].page.y);
-            layer_info[i].page.width=(ReadBlobMSBLong(image)-layer_info[i].page.x);
+            layer_info[i].page.height=
+              ReadBlobMSBLong(image)-layer_info[i].page.y;
+            layer_info[i].page.width=
+              ReadBlobMSBLong(image)-layer_info[i].page.x;
             layer_info[i].channels=ReadBlobMSBShort(image);
-          if (image->debug != MagickFalse)
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),"    offset(%ld,%ld), size(%ld,%ld), channels=%d",
-              layer_info[i].page.x, layer_info[i].page.y,
-              layer_info[i].page.height, layer_info[i].page.width,
-              layer_info[i].channels);
+            if (layer_info[i].channels > MaxPSDChannels)
+              ThrowReaderException(CorruptImageError,"MaximumChannelsExceeded");
+            if (image->debug != MagickFalse)
+              (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                "    offset(%ld,%ld), size(%ld,%ld), channels=%d",
+                layer_info[i].page.x, layer_info[i].page.y,
+                layer_info[i].page.height, layer_info[i].page.width,
+                layer_info[i].channels);
             for (j=0; j < (long) layer_info[i].channels; j++)
             {
               layer_info[i].channel_info[j].type=(short)
                 ReadBlobMSBShort(image);
               layer_info[i].channel_info[j].size=ReadBlobMSBLong(image);
-
-         if (image->debug != MagickFalse)
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),"    channel[%ld]: type=%d, size=%ld",
-               j, layer_info[i].channel_info[j].type,
-               (long) layer_info[i].channel_info[j].size);
+              if (image->debug != MagickFalse)
+                (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                  "    channel[%ld]: type=%d, size=%ld",j,
+                  layer_info[i].channel_info[j].type,
+              (long) layer_info[i].channel_info[j].size);
             }
             count=ReadBlob(image,4,(unsigned char *) type);
-            if ((count == 0) || (LocaleNCompare(type,"8BIM",4) != 0)) {
-          if (image->debug != MagickFalse)
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  layer type was %.4s instead of 8BIM", type);
-              ThrowReaderException(CorruptImageError,"ImproperImageHeader");
-        }
+            if ((count == 0) || (LocaleNCompare(type,"8BIM",4) != 0))
+              {
+                if (image->debug != MagickFalse)
+                  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                    "  layer type was %.4s instead of 8BIM", type);
+                 ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+              }
             (void) ReadBlob(image,4,(unsigned char *) layer_info[i].blendkey);
             layer_info[i].opacity=(Quantum)
               (MaxRGB-ScaleCharToQuantum(ReadBlobByte(image)));
             layer_info[i].clipping=(unsigned char) ReadBlobByte(image);
             layer_info[i].flags=(unsigned char) ReadBlobByte(image);
             layer_info[i].visible=!(layer_info[i].flags & 0x02);
-          if (image->debug != MagickFalse)
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),"    blend=%.4s, opacity=%d, clipping=%s, flags=%d, visible=%s",
-              layer_info[i].blendkey, layer_info[i].opacity,
-              layer_info[i].clipping ? "true" : "false",
-              layer_info[i].flags,
-              layer_info[i].visible ? "true" : "false");
-
-        (void) ReadBlobByte(image);  /* filler */
-        combinedlength = 0;
+            if (image->debug != MagickFalse)
+              (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                "    blend=%.4s, opacity=%d, clipping=%s, flags=%d, visible=%s",
+                layer_info[i].blendkey, layer_info[i].opacity,
+                layer_info[i].clipping ? "true" : "false",
+                layer_info[i].flags,
+                layer_info[i].visible ? "true" : "false");
+            (void) ReadBlobByte(image);  /* filler */
+            combinedlength=0;
             size=ReadBlobMSBLong(image);
             if (size != 0)
               {
-            if (image->debug != MagickFalse)
-              (void) LogMagickEvent(CoderEvent,GetMagickModule(),"    layer contains additional info");
+                if (image->debug != MagickFalse)
+                  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                    "    layer contains additional info");
                 length=ReadBlobMSBLong(image);
                 if (length != 0)
                   {
@@ -903,12 +898,12 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
                       (ReadBlobMSBLong(image)-layer_info[i].mask.y);
                     layer_info[i].mask.width=
                       (ReadBlobMSBLong(image)-layer_info[i].mask.x);
-
-              if (image->debug != MagickFalse)
-                (void) LogMagickEvent(CoderEvent,GetMagickModule(),"      layer mask: offset(%ld,%ld), size(%ld,%ld), length=%ld",
-                  layer_info[i].mask.x, layer_info[i].mask.y,
-                  layer_info[i].mask.width, layer_info[i].mask.height,
-                  (long) length-16);
+                    if (image->debug != MagickFalse)
+                      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                        "      layer mask: offset(%ld,%ld), size(%ld,%ld), length=%ld",
+                        layer_info[i].mask.x, layer_info[i].mask.y,
+                        layer_info[i].mask.width, layer_info[i].mask.height,
+                        (long) length-16);
                     /*
                       Skip over the rest of the layer mask information.
                     */
@@ -916,44 +911,43 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
                       (void) ReadBlobByte(image);
                   }
                 combinedlength += length + 4;  /* +4 for length */
-
                 length=ReadBlobMSBLong(image);
                 if (length != 0)
                   {
                     /*
                       Layer blending ranges info.
                     */
-              if (image->debug != MagickFalse)
-                (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                  "      layer blending ranges: length=%ld",
-                  (long) length);
-
+                    if (image->debug != MagickFalse)
+                      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                        "      layer blending ranges: length=%ld",
+                        (long) length);
                     /*
-                      we read it, but don't use it...
+                      We read it, but don't use it...
                     */
-                    for (j=0; j < (long) (length); j+=8) {
+                    for (j=0; j < (long) (length); j+=8)
+                    {
                       size_t blend_source = ReadBlobMSBLong(image);
                       size_t blend_dest = ReadBlobMSBLong(image);
                       if (image->debug != MagickFalse)
                         (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                          "        source(%x), dest(%x)",
-                          (unsigned int) blend_source,(unsigned int) blend_dest);
+                          "        source(%x), dest(%x)",(unsigned int)
+                          blend_source,(unsigned int) blend_dest);
                     }
                   }
-                combinedlength += length + 4;  /* +4 for length */
-
+                combinedlength+=length + 4;  /* +4 for length */
                 length=(size_t) ReadBlobByte(image);
                 if (length != 0)
-            {
-            /* layer name */
-            for (j=0; j < (long) (length); j++)
-              layer_info[i].name[j]=(unsigned char) ReadBlobByte(image);
-            layer_info[i].name[j] = 0;  /* zero term */
-
-            if (image->debug != MagickFalse)
-              (void) LogMagickEvent(CoderEvent,GetMagickModule(),"      layer name: %s",
-                layer_info[i].name);
-            }
+                  {
+                    /*
+                      Layer name.
+                    */
+                    for (j=0; j < (long) (length); j++)
+                      layer_info[i].name[j]=(unsigned char) ReadBlobByte(image);
+                    layer_info[i].name[j]='\0';
+                    if (image->debug != MagickFalse)
+                      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                        "      layer name: %s",layer_info[i].name);
+                 }
 #if     0
           if ( length == 0 )
             padBytes = 3;
@@ -965,7 +959,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
           }
                 combinedlength += length + padBytes + 1;  /* +1 for length */
 #else
-                combinedlength += length + 1;  /* +1 for length */
+               combinedlength+=length + 1;  /* +1 for length */
 #endif
 
 #if     0  /* still in development */
@@ -1007,9 +1001,10 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
                /*
                   Skip the rest of the variable data until we support it.
                 */
-            if (image->debug != MagickFalse)
-              (void) LogMagickEvent(CoderEvent,GetMagickModule(),"      unsupported data: length=%ld",
-                (long) size-combinedlength);
+                if (image->debug != MagickFalse)
+                  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                    "      unsupported data: length=%ld",(long)
+                    size-combinedlength);
                 for (j=0; j < (long) (size-combinedlength); j++)
                   (void) ReadBlobByte(image);
               }
@@ -1017,20 +1012,20 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
               Allocate layered image.
             */
             layer_info[i].image=CloneImage(image,layer_info[i].page.width,
-                                           layer_info[i].page.height,
-                                           MagickTrue,&image->exception);
+              layer_info[i].page.height,MagickTrue,&image->exception);
             if (layer_info[i].image == (Image *) NULL)
               {
                 for (j=0; j < i; j++)
                   layer_info[j].image=DestroyImage(layer_info[j].image);
-
-            if (image->debug != MagickFalse)
-              (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  allocation of image for layer %ld failed", i);
-
-                ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+                if (image->debug != MagickFalse)
+                  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                    "  allocation of image for layer %ld failed", i);
+                ThrowReaderException(ResourceLimitError,
+                  "MemoryAllocationFailed");
               }
-          if (image->debug != MagickFalse)
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),"    setting up new layer image");
+            if (image->debug != MagickFalse)
+              (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                "    setting up new layer image");
             SetImage(layer_info[i].image,OpaqueOpacity);
             layer_info[i].image->compose=
               PSDBlendModeToCompositeOperator(layer_info[i].blendkey);
@@ -1041,41 +1036,43 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
             for (j=0; j < (long) layer_info[i].channels; j++)
               if (layer_info[i].channel_info[j].type == -1)
                 layer_info[i].image->matte=MagickTrue;
-
-            /* set up some hidden attributes for folks that need them */
-        FormatMagickString(s,MaxTextExtent,"%ld", layer_info[i].page.x );
-        (void) SetImageAttribute(layer_info[i].image,"[layer-xpos]",s);
-        FormatMagickString(s,MaxTextExtent,"%ld", layer_info[i].page.y );
-        (void) SetImageAttribute(layer_info[i].image,"[layer-ypos]",s);
-        FormatMagickString(s,MaxTextExtent,"%d", layer_info[i].opacity );
-        (void) SetImageAttribute(layer_info[i].image,"[layer-opacity]",s);
-        (void) SetImageAttribute(layer_info[i].image,"[layer-name]",
-          (char *) layer_info[i].name);
-      }
-
-
+            /*
+              Set up some hidden attributes for folks that need them.
+            */
+            FormatMagickString(s,MaxTextExtent,"%ld", layer_info[i].page.x );
+            (void) SetImageAttribute(layer_info[i].image,"[layer-xpos]",s);
+            FormatMagickString(s,MaxTextExtent,"%ld", layer_info[i].page.y );
+            (void) SetImageAttribute(layer_info[i].image,"[layer-ypos]",s);
+            FormatMagickString(s,MaxTextExtent,"%d", layer_info[i].opacity );
+            (void) SetImageAttribute(layer_info[i].image,"[layer-opacity]",s);
+            (void) SetImageAttribute(layer_info[i].image,"[layer-name]",
+              (char *) layer_info[i].name);
+          }
         if (image->debug != MagickFalse)
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  reading image data for layers");
-          /*
-            Read pixel data for each layer.
-          */
-          for (i=0; i < number_layers; i++)
-          {
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+            "  reading image data for layers");
+        /*
+          Read pixel data for each layer.
+        */
+        for (i=0; i < number_layers; i++)
+        {
           if (image->debug != MagickFalse)
-              (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  reading data for layer %ld", i);
+            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+              "  reading data for layer %ld",i);
             for (j=0; j < (long) layer_info[i].channels; j++)
             {
-          if (image->debug != MagickFalse)
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),"    reading data for channel %ld", j);
-#if     1
+              if (image->debug != MagickFalse)
+                (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                  "    reading data for channel %ld", j);
+#if 1
               if (layer_info[i].channel_info[j].size <= (2*layer_info[i].image->rows))
                 {
                   long
                     k;
 
-      if (image->debug != MagickFalse)
-              (void) LogMagickEvent(CoderEvent,GetMagickModule(),"      layer data is empty");
-
+                  if (image->debug != MagickFalse)
+                    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                      "      layer data is empty");
                   /*
                     A layer without data.
                   */
@@ -1090,21 +1087,21 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   /*
                     Read RLE compressed data.
                   */
-          if (image->debug != MagickFalse)
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),"      layer data is RLE compressed");
-
+                  if (image->debug != MagickFalse)
+                    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                      "      layer data is RLE compressed");
                   for (y=0; y < (long) layer_info[i].image->rows; y++)
                     (void) ReadBlobMSBShort(layer_info[i].image);
                   (void) DecodeImage(layer_info[i].image,
                      layer_info[i].channel_info[j].type);
                   continue;
                 }
-
               /*
                 Read uncompressed pixel data as separate planes.
               */
-        if (image->debug != MagickFalse)
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"      layer data is uncompressed");
+              if (image->debug != MagickFalse)
+                (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                  "      layer data is uncompressed");
               packet_size=1;
               if (layer_info[i].image->storage_class == PseudoClass)
                 {
@@ -1126,8 +1123,8 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   if (packet_size == 1)
                     pixel=ScaleCharToQuantum(ReadBlobByte(layer_info[i].image));
                   else
-                    pixel=
-                      ScaleShortToQuantum(ReadBlobMSBShort(layer_info[i].image));
+                    pixel=ScaleShortToQuantum(ReadBlobMSBShort(
+                      layer_info[i].image));
                   switch (layer_info[i].channel_info[j].type)
                   {
                     case -1:  /* transparency mask */
@@ -1210,8 +1207,9 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 /*
                   Correct CMYK levels.
                 */
-            if (image->debug != MagickFalse)
-              (void) LogMagickEvent(CoderEvent,GetMagickModule(),"    correcting CMYK values");
+                if (image->debug != MagickFalse)
+                  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                    "    correcting CMYK values");
                 for (y=0; y < (long) layer_info[i].image->rows; y++)
                 {
                   q=GetImagePixels(layer_info[i].image,0,y,
@@ -1233,14 +1231,13 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
               }
           }
         diff_offset = TellBlob(image) - offset;
-
         /* added by palf -> invisible group layer make layer of this group
            invisible I consider that all layer with width and height null are
            layer for group layer */
        {
          short inside_layer = 0;
          short layer_visible = 0;
-       for (i=number_layers-1; i >=0; i--)
+         for (i=number_layers-1; i >=0; i--)
          {
            if ((layer_info[i].page.width == 0) || (layer_info[i].page.height == 0))
              {
@@ -1256,17 +1253,16 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
              }
            else if ((inside_layer == 1) && (layer_visible == 0))
              {
-          layer_info[i].visible = (short int) layer_visible;
-          layer_info[i].image->compose=NoCompositeOp;
+               layer_info[i].visible = (unsigned char) layer_visible;
+               layer_info[i].image->compose=NoCompositeOp;
              }
          }
        }
-
        /* added by palf -> suppression of empty layer */
        /* I consider that all layer with width and height null are layer for group layer */
        for (i=0; i < number_layers; i++)
-         {
-           if ((layer_info[i].page.width == 0) || (layer_info[i].page.height == 0))
+       {
+         if ((layer_info[i].page.width == 0) || (layer_info[i].page.height == 0))
              {
           if (layer_info[i].image != (Image *) NULL)
             layer_info[i].image=DestroyImage(layer_info[i].image);
@@ -1307,10 +1303,7 @@ static Image *ReadPSDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
               return(returnImage);
             }
-        } else {
-        if (image->debug != MagickFalse)
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"  image has no layers");
-      }
+        }
     }
 
   /*
@@ -1529,8 +1522,8 @@ ModuleExport void UnregisterPSDImage(void)
 %
 */
 
-static void WriteOneChannel( Image* image, Image* tmp_image,
-               unsigned char *pixels, QuantumType whichQuantum )
+static void WriteOneChannel(Image *image,Image *tmp_image,unsigned char *pixels,
+  QuantumType whichQuantum)
 {
   int
     y;
@@ -1549,36 +1542,35 @@ static void WriteOneChannel( Image* image, Image* tmp_image,
     p=AcquireImagePixels(tmp_image,0,y,tmp_image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
     break;
-    (void) PopImagePixels(tmp_image,whichQuantum,pixels);
+    (void) ImportQuantumPixels(tmp_image,whichQuantum,0,pixels);
     (void) WriteBlob(image,packet_size*tmp_image->columns,pixels);
   }
 }
 
-static void WriteImageChannels( Image* image, Image* tmp_image, unsigned char *pixels )
+static void WriteImageChannels(Image* image,Image *tmp_image,
+  unsigned char *pixels )
 {
   /*
-  Write uncompressed pixel data as separate planes.
+    Write uncompressed pixel data as separate planes.
   */
   (void) WriteBlobMSBShort(image,0);  /* compression */
   if (tmp_image->storage_class == PseudoClass)
-  {
-    if (tmp_image->matte == MagickFalse)
-      WriteOneChannel( image, tmp_image, pixels, IndexQuantum );
-    else
-      WriteOneChannel( image, tmp_image, pixels, IndexAlphaQuantum );
-  }
+    {
+      if (tmp_image->matte == MagickFalse)
+        WriteOneChannel(image,tmp_image,pixels,IndexQuantum);
+      else
+        WriteOneChannel(image,tmp_image,pixels,IndexAlphaQuantum);
+    }
   else
-  {
-    if (tmp_image->matte)
-      WriteOneChannel( image, tmp_image, pixels, AlphaQuantum );
-
-      WriteOneChannel( image, tmp_image, pixels, RedQuantum );
-    WriteOneChannel( image, tmp_image, pixels, GreenQuantum );
-    WriteOneChannel( image, tmp_image, pixels, BlueQuantum );
-
-    if (tmp_image->colorspace == CMYKColorspace)
-    WriteOneChannel( image, tmp_image, pixels, BlackQuantum );
-  }
+    {
+      if (tmp_image->matte)
+        WriteOneChannel(image,tmp_image,pixels,AlphaQuantum);
+      WriteOneChannel(image,tmp_image,pixels,RedQuantum);
+      WriteOneChannel(image,tmp_image,pixels,GreenQuantum);
+      WriteOneChannel(image,tmp_image,pixels,BlueQuantum);
+      if (tmp_image->colorspace == CMYKColorspace)
+        WriteOneChannel(image,tmp_image,pixels,BlackQuantum);
+    }
 }
 
 /* Write white background, RLE-compressed */
