@@ -197,8 +197,8 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
   /*
     Set the page geometry.
   */
-  dx_resolution=72.0;
-  dy_resolution=72.0;
+  dx_resolution=DefaultResolution;
+  dy_resolution=DefaultResolution;
   if ((image->x_resolution == 0.0) || (image->y_resolution == 0.0))
     {
       GeometryInfo
