@@ -196,7 +196,8 @@ static Image *ReadVIDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   read_info=DestroyImageInfo(read_info);
   filelist=(char **) RelinquishMagickMemory(filelist);
   if (images == (Image *) NULL)
-    ThrowReaderException(CorruptImageError,"UnableToReadVIDImage");
+    ThrowReaderException(CorruptImageError,
+      "ImageFileDoesNotContainAnyImageData");
   /*
     Create the visual image directory.
   */
@@ -206,8 +207,6 @@ static Image *ReadVIDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   montage_image=MontageImages(GetFirstImageInList(images),montage_info,
     exception);
   montage_info=DestroyMontageInfo(montage_info);
-  if (montage_image == (Image *) NULL)
-    ThrowReaderException(CorruptImageError,"UnableToReadVIDImage");
   images=DestroyImageList(images);
   return(montage_image);
 }
