@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,11 +42,14 @@
 #include "magick/studio.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
 #include "magick/color.h"
 #include "magick/color_private.h"
 #include "magick/colorspace.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/image.h"
+#include "magick/image_private.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
@@ -57,7 +60,7 @@
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteVIFFImage(const ImageInfo *,Image *);
 
 /*
@@ -71,17 +74,15 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsVIFF() returns True if the image format type, identified by the
+%  IsVIFF() returns MagickTrue if the image format type, identified by the
 %  magick string, is VIFF.
 %
 %  The format of the IsVIFF method is:
 %
-%      unsigned int IsVIFF(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsVIFF(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsVIFF() returns True if the image format type is VIFF.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -89,13 +90,13 @@ static unsigned int
 %
 %
 */
-static unsigned int IsVIFF(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsVIFF(const unsigned char *magick,const size_t length)
 {
   if (length < 2)
-    return(False);
+    return(MagickFalse);
   if (memcmp(magick,"\253\001",2) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -213,6 +214,9 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
   long
     y;
 
+  MagickBooleanType
+    status;
+
   MagickSizeType
     number_pixels;
 
@@ -238,9 +242,6 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
     buffer[7],
     *viff_pixels;
 
-  unsigned int
-    status;
-
   unsigned long
     bytes_per_pixel,
     lsb_first,
@@ -254,15 +255,15 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
-  if (image_info->debug != False)
+  if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
+  if (status == MagickFalse)
     {
-      DestroyImageList(image);
+      image=DestroyImageList(image);
       return((Image *) NULL);
     }
   /*
@@ -287,7 +288,7 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
     (void) ReadBlob(image,512,(unsigned char *) viff_info.comment);
     viff_info.comment[511]='\0';
     if (strlen(viff_info.comment) > 4)
-      (void) SetImageAttribute(image,"comment",viff_info.comment);
+      (void) SetImageAttribute(image,"Comment",viff_info.comment);
     if ((viff_info.machine_dependency == VFF_DEP_DECORDER) ||
         (viff_info.machine_dependency == VFF_DEP_NSORDER))
       {
@@ -388,7 +389,7 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
             image->colors=image->depth <= 8 ? 256 : 65536L;
             if (viff_info.data_storage_type == VFF_TYP_BIT)
               image->colors=2;
-            if (AllocateImageColormap(image,image->colors) == False)
+            if (AllocateImageColormap(image,image->colors) == MagickFalse)
               ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           }
         break;
@@ -412,7 +413,7 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
           default: bytes_per_pixel=1; break;
         }
         image->colors=viff_info.map_columns;
-        if (AllocateImageColormap(image,image->colors) == False)
+        if (AllocateImageColormap(image,image->colors) == MagickFalse)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         viff_colormap=(unsigned char *)
           AcquireMagickMemory(bytes_per_pixel*image->colors*viff_info.map_rows);
@@ -479,12 +480,12 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
     /*
       Initialize image structure.
     */
-    image->matte=(viff_info.number_data_bands == 4);
+    image->matte=viff_info.number_data_bands == 4 ? MagickTrue : MagickFalse;
     image->storage_class=
       (viff_info.number_data_bands < 3 ? PseudoClass : DirectClass);
     image->columns=viff_info.rows;
     image->rows=viff_info.columns;
-    if ((image_info->ping != False) && (image_info->number_scenes != 0))
+    if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     /*
@@ -637,12 +638,17 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
                   ((*p) & (0x01 << bit) ? !polarity : polarity);
               p++;
             }
-          if (SyncImagePixels(image) == False)
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
-              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == False)
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
         }
       }
     else
@@ -655,12 +661,17 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
           indexes=GetIndexes(image);
           for (x=0; x < (long) image->columns; x++)
             indexes[x]=(*p++);
-          if (SyncImagePixels(image) == False)
+          if (SyncImagePixels(image) == MagickFalse)
             break;
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
-              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == False)
-                break;
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
+              {
+                status=image->progress_monitor(LoadImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
+                  break;
+              }
         }
       else
         {
@@ -689,13 +700,15 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
               p++;
               q++;
             }
-            if (SyncImagePixels(image) == False)
+            if (SyncImagePixels(image) == MagickFalse)
               break;
             if (image->previous == (Image *) NULL)
-              if (QuantumTick(y,image->rows))
+              if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                  (QuantumTick(y,image->rows) != MagickFalse))
                 {
-                  status=MagickMonitor(LoadImageTag,y,image->rows,exception);
-                  if (status == False)
+                  status=image->progress_monitor(LoadImageTag,y,image->rows,
+                    image->client_data);
+                  if (status == MagickFalse)
                     break;
                 }
           }
@@ -703,7 +716,7 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
     viff_pixels=(unsigned char *) RelinquishMagickMemory(viff_pixels);
     if (image->storage_class == PseudoClass)
       (void) SyncImage(image);
-    if (EOFBlob(image) != False)
+    if (EOFBlob(image) != MagickFalse)
       {
         ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
           image->filename);
@@ -724,14 +737,17 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
         AllocateNextImage(image_info,image);
         if (image->next == (Image *) NULL)
           {
-            DestroyImageList(image);
+            image=DestroyImageList(image);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
-        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
-          exception);
-        if (status == False)
-          break;
+        if (image->progress_monitor != (MagickProgressMonitor) NULL)
+          {
+            status=image->progress_monitor(LoadImagesTag,TellBlob(image),
+              GetBlobSize(image),image->client_data);
+            if (status == MagickFalse)
+              break;
+          }
       }
   } while ((count != 0) && (viff_info.identifier == 0xab));
   CloseBlob(image);
@@ -821,21 +837,19 @@ ModuleExport void UnregisterVIFFImage(void)
 %
 %  The format of the WriteVIFFImage method is:
 %
-%      unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteVIFFImage(const ImageInfo *image_info,
+%        Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteVIFFImage return True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
-%
 %    o image_info: The image info.
 %
 %    o image:  The image.
 %
 %
 */
-static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteVIFFImage(const ImageInfo *image_info,
+  Image *image)
 {
 #define VFF_CM_genericRGB  15
 #define VFF_CM_NONE  0
@@ -896,6 +910,9 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
   long
     y;
 
+  MagickBooleanType
+    status;
+
   MagickOffsetType
     scene;
 
@@ -922,9 +939,6 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
     buffer[8],
     *viff_pixels;
 
-  unsigned int
-    status;
-
   ViffInfo
     viff_info;
 
@@ -935,10 +949,10 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
-  if (status == False)
+  if (status == MagickFalse)
     return(status);
   (void) ResetMagickMemory(&viff_info,0,sizeof(ViffInfo));
   scene=0;
@@ -954,13 +968,10 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
     viff_info.version=3;
     viff_info.machine_dependency=VFF_DEP_IEEEORDER;  /* IEEE byte ordering */
     *viff_info.comment='\0';
-    attribute=GetImageAttribute(image,"comment");
+    attribute=GetImageAttribute(image,"Comment");
     if (attribute != (const ImageAttribute *) NULL)
-      {
-        (void) strncpy(viff_info.comment,attribute->value,
-          Min(strlen(attribute->value),511));
-        viff_info.comment[Min(strlen(attribute->value),511)]='\0';
-      }
+      (void) CopyMagickString(viff_info.comment,attribute->value,
+        Min(strlen(attribute->value),511)+1);
     viff_info.rows=image->columns;
     viff_info.columns=image->rows;
     viff_info.subrows=0;
@@ -996,7 +1007,7 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
         viff_info.color_space_model=VFF_CM_NONE;
         viff_info.data_storage_type=VFF_TYP_1_BYTE;
         packets=number_pixels;
-        if (IsGrayImage(image,&image->exception) == False)
+        if (IsGrayImage(image,&image->exception) == MagickFalse)
           {
             /*
               Colormapped VIFF raster.
@@ -1077,23 +1088,24 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
             *q=ScaleQuantumToChar(p->red);
             *(q+number_pixels)=ScaleQuantumToChar(p->green);
             *(q+number_pixels*2)=ScaleQuantumToChar(p->blue);
-            if (image->matte != False)
+            if (image->matte != MagickFalse)
               *(q+number_pixels*3)=ScaleQuantumToChar(MaxRGB-p->opacity);
             p++;
             q++;
           }
           if (image->previous == (Image *) NULL)
-            if (QuantumTick(y,image->rows))
+            if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                (QuantumTick(y,image->rows) != MagickFalse))
               {
-                status=MagickMonitor(SaveImageTag,y,image->rows,
-                  &image->exception);
-                if (status == False)
+                status=image->progress_monitor(SaveImageTag,y,image->rows,
+                  image->client_data);
+                if (status == MagickFalse)
                   break;
               }
         }
       }
     else
-      if (IsGrayImage(image,&image->exception) == False)
+      if (IsGrayImage(image,&image->exception) == MagickFalse)
         {
           unsigned char
             *viff_colormap;
@@ -1126,11 +1138,12 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
             for (x=0; x < (long) image->columns; x++)
               *q++=indexes[x];
             if (image->previous == (Image *) NULL)
-              if (QuantumTick(y,image->rows))
+              if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                  (QuantumTick(y,image->rows) != MagickFalse))
                 {
-                  status=MagickMonitor(SaveImageTag,y,image->rows,
-                    &image->exception);
-                  if (status == False)
+                  status=image->progress_monitor(SaveImageTag,y,image->rows,
+                    image->client_data);
+                  if (status == MagickFalse)
                     break;
                 }
           }
@@ -1180,11 +1193,12 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
               if (bit != 0)
                 *q++=byte >> (8-bit);
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows))
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
                   {
-                    status=MagickMonitor(SaveImageTag,y,image->rows,
-                      &image->exception);
-                    if (status == False)
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
+                    if (status == MagickFalse)
                       break;
                   }
             }
@@ -1206,11 +1220,12 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
                 p++;
               }
               if (image->previous == (Image *) NULL)
-                if (QuantumTick(y,image->rows))
+                if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+                    (QuantumTick(y,image->rows) != MagickFalse))
                   {
-                    status=MagickMonitor(SaveImageTag,y,image->rows,
-                      &image->exception);
-                    if (status == False)
+                    status=image->progress_monitor(SaveImageTag,y,image->rows,
+                      image->client_data);
+                    if (status == MagickFalse)
                       break;
                   }
             }
@@ -1220,11 +1235,15 @@ static unsigned int WriteVIFFImage(const ImageInfo *image_info,Image *image)
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
-      &image->exception);
-    if (status == False)
-      break;
-  } while (image_info->adjoin != False);
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
+  } while (image_info->adjoin != MagickFalse);
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
