@@ -506,7 +506,7 @@ MagickExport MagickBooleanType EqualizeImage(Image *image)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Use GammaImage() to gamma-correct a particular image channel.  The same
+%  GammaImage() gamma-corrects a particular image channel.  The same
 %  image viewed on different devices will have perceptual differences in the
 %  way the image's intensities are represented on the screen.  Specify
 %  individual gamma levels for the red, green, and blue channels, or adjust
