@@ -17,7 +17,7 @@
 %                               December 2002                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,9 +41,11 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/error.h"
-#include "magick/list.h"
 #include "magick/blob.h"
+#include "magick/blob_private.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
+#include "magick/list.h"
 #include "magick/memory_.h"
 #include "magick/string_.h"
 
@@ -81,7 +83,7 @@ MagickExport void AppendImageToList(Image **images,const Image *image)
   if (image == (Image *) NULL)
     return;
   assert(image->signature == MagickSignature);
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   if ((*images) == (Image *) NULL)
     {
@@ -137,11 +139,11 @@ MagickExport Image *CloneImageList(const Image *images,ExceptionInfo *exception)
   image=(Image *) NULL;
   for (p=(Image *) NULL; images != (Image *) NULL; images=images->next)
   {
-    clone=CloneImage(images,0,0,True,exception);
+    clone=CloneImage(images,0,0,MagickTrue,exception);
     if (clone == (Image *) NULL)
       {
         if (image != (Image *) NULL)
-          DestroyImageList(image);
+          image=DestroyImageList(image);
         return((Image *) NULL);
       }
     if (image == (Image *) NULL)
@@ -189,7 +191,7 @@ MagickExport void DeleteImageFromList(Image **images)
   if ((*images) == (Image *) NULL)
     return;
   assert((*images)->signature == MagickSignature);
-  if ((*images)->debug != False)
+  if ((*images)->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),(*images)->filename);
   p=(*images);
   if ((p->previous == (Image *) NULL) && (p->next == (Image *) NULL))
@@ -207,7 +209,7 @@ MagickExport void DeleteImageFromList(Image **images)
           *images=p->next;
         }
     }
-  DestroyImage(p);
+  p=DestroyImage(p);
 }
 
 /*
@@ -225,7 +227,7 @@ MagickExport void DeleteImageFromList(Image **images)
 %
 %  The format of the DestroyImageList method is:
 %
-%      void DestroyImageList(Image *image)
+%      Image *DestroyImageList(Image *image)
 %
 %  A description of each parameter follows:
 %
@@ -233,15 +235,15 @@ MagickExport void DeleteImageFromList(Image **images)
 %
 %
 */
-MagickExport void DestroyImageList(Image *images)
+MagickExport Image *DestroyImageList(Image *images)
 {
   register Image
     *p;
 
   if (images == (Image *) NULL)
-    return;
+    return((Image *) NULL);
   assert(images->signature == MagickSignature);
-  if (images->debug != False)
+  if (images->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),images->filename);
   for (p=images; p->previous != (Image *) NULL; p=p->previous);
   for (images=p; p != (Image *) NULL; images=p)
@@ -250,8 +252,9 @@ MagickExport void DestroyImageList(Image *images)
     if (p != (Image *) NULL)
       p->previous=(Image *) NULL;
     images->next=(Image *) NULL;
-    DestroyImage(images);
+    images=DestroyImage(images);
   }
+  return((Image *) NULL);
 }
 
 /*
@@ -324,7 +327,7 @@ MagickExport Image *GetImageFromList(const Image *images,const long index)
   if (images == (Image *) NULL)
     return((Image *) NULL);
   assert(images->signature == MagickSignature);
-  if (images->debug != False)
+  if (images->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),images->filename);
   for (p=images; p->previous != (Image *) NULL; p=p->previous);
   for (i=0; p != (Image *) NULL; p=p->next)
@@ -403,7 +406,7 @@ MagickExport unsigned long GetImageListLength(const Image *images)
   if (images == (Image *) NULL)
     return(0);
   assert(images->signature == MagickSignature);
-  if (images->debug != False)
+  if (images->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),images->filename);
   while (images->previous != (Image *) NULL)
     images=images->previous;
@@ -474,7 +477,7 @@ MagickExport Image *GetNextImageInList(const Image *images)
   if (images == (Image *) NULL)
     return((Image *) NULL);
   assert(images->signature == MagickSignature);
-  if (images->debug != False)
+  if (images->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),images->filename);
   return(images->next);
 }
@@ -551,10 +554,10 @@ MagickExport Image **ImageListToArray(const Image *images,
   if (images == (Image *) NULL)
     return((Image **) NULL);
   assert(images->signature == MagickSignature);
-  if (images->debug != False)
+  if (images->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),images->filename);
-  group=(Image **) AcquireMagickMemory(
-    (size_t) (GetImageListLength(images)+1)*sizeof(Image *));
+  group=(Image **) AcquireMagickMemory((size_t)
+    (GetImageListLength(images)+1)*sizeof(*group));
   if (group == (Image **) NULL)
     {
       (void) ThrowMagickException(exception,GetMagickModule(),
@@ -602,7 +605,7 @@ MagickExport void InsertImageInList(Image **images,Image *image)
   assert(images != (Image **) NULL);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   if ((*images) == (Image *) NULL)
     return;
@@ -699,7 +702,7 @@ MagickExport Image *RemoveImageFromList(Image **images)
   if ((*images) == (Image *) NULL)
     return((Image *) NULL);
   assert((*images)->signature == MagickSignature);
-  if ((*images)->debug != False)
+  if ((*images)->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),(*images)->filename);
   p=(*images);
   if ((p->previous == (Image *) NULL) && (p->next == (Image *) NULL))
@@ -752,7 +755,7 @@ MagickExport Image *RemoveFirstImageFromList(Image **images)
   if ((*images) == (Image *) NULL)
     return((Image *) NULL);
   assert((*images)->signature == MagickSignature);
-  if ((*images)->debug != False)
+  if ((*images)->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),(*images)->filename);
   image=(*images);
   while (image->previous != (Image *) NULL)
@@ -799,7 +802,7 @@ MagickExport Image *RemoveLastImageFromList(Image **images)
   if ((*images) == (Image *) NULL)
     return((Image *) NULL);
   assert((*images)->signature == MagickSignature);
-  if ((*images)->debug != False)
+  if ((*images)->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),(*images)->filename);
   image=(*images);
   while (image->next != (Image *) NULL)
@@ -844,7 +847,7 @@ MagickExport void ReplaceImageInList(Image **images,Image *image)
   assert(images != (Image **) NULL);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  if (image->debug != False)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
   if ((*images) == (Image *) NULL)
     return;
@@ -855,7 +858,7 @@ MagickExport void ReplaceImageInList(Image **images,Image *image)
   image->previous=(*images)->previous;
   if (image->previous != (Image *) NULL)
     image->previous->next=image;
-  DestroyImage(*images);
+  *images=DestroyImage(*images);
   (*images)=image;
 }
 
@@ -894,7 +897,7 @@ MagickExport void ReverseImageList(Image **images)
   if ((*images) == (Image *) NULL)
     return;
   assert((*images)->signature == MagickSignature);
-  if ((*images)->debug != False)
+  if ((*images)->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),(*images)->filename);
   for (p=(*images); p->next != (Image *) NULL; p=p->next);
   *images=p;
@@ -950,7 +953,7 @@ MagickExport void SpliceImageIntoList(Image **images,const unsigned long length,
   if ((*images) == (Image *) NULL)
     return;
   assert((*images)->signature == MagickSignature);
-  if ((*images)->debug != False)
+  if ((*images)->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),(*images)->filename);
   split=SplitImageList(*images);
   if (split == (Image *) NULL)
@@ -1076,5 +1079,6 @@ MagickExport Image *SyncNextImageInList(const Image *images)
       DestroyBlob(images->next);
       images->next->blob=ReferenceBlob(images->blob);
     }
+  images->next->endian=images->endian;
   return(images->next);
 }
