@@ -17,7 +17,7 @@
 %                                October 2001                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -40,7 +40,8 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/error.h"
+#include "magick/exception.h"
+#include "magick/exception_private.h"
 #include "magick/hashmap.h"
 #include "magick/image.h"
 #include "magick/list.h"
@@ -99,12 +100,12 @@ static void
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  DeleteMagickRegistry() deletes an entry in the registry as defined by the id.
-%  It returns True if the entry is deleted otherwise False if no entry is found
+%  It returns MagickTrue if the entry is deleted otherwise MagickFalse if no entry is found
 %  in the registry that matches the id.
 %
 %  The format of the DeleteMagickRegistry method is:
 %
-%      unsigned int DeleteMagickRegistry(const long id)
+%      MagickBooleanType DeleteMagickRegistry(const long id)
 %
 %  A description of each parameter follows:
 %
@@ -112,16 +113,16 @@ static void
 %
 %
 */
-MagickExport unsigned int DeleteMagickRegistry(const long id)
+MagickExport MagickBooleanType DeleteMagickRegistry(const long id)
 {
   register const RegistryInfo
     *p;
 
   if (registry_list == (LinkedListInfo *) NULL)
-    return(False);
-  if (IsLinkedListEmpty(registry_list) == True)
-    return(False);
-  AcquireSemaphoreInfo(&registry_semaphore);
+    return(MagickFalse);
+  if (IsLinkedListEmpty(registry_list) == MagickTrue)
+    return(MagickFalse);
+  registry_semaphore=AcquireSemaphoreInfo(registry_semaphore);
   ResetLinkedListIterator(registry_list);
   p=(const RegistryInfo *) GetNextElementInLinkedList(registry_list);
   while (p != (const RegistryInfo *) NULL)
@@ -139,8 +140,8 @@ MagickExport unsigned int DeleteMagickRegistry(const long id)
         RemoveElementByValueFromLinkedList(registry_list,p);
       (void) DestroyRegistryElement(registry_info);
     }
-  RelinquishSemaphoreInfo(&registry_semaphore);
-  return((unsigned int) (p != (RegistryInfo *) NULL));
+  RelinquishSemaphoreInfo(registry_semaphore);
+  return((MagickBooleanType) (p != (RegistryInfo *) NULL));
 }
 
 /*
@@ -173,12 +174,12 @@ static void *DestroyRegistryElement(void *registry_info)
   {
     case ImageRegistryType:
     {
-      DestroyImage((Image *) p->blob);
+      p->blob=DestroyImage((Image *) p->blob);
       break;
     }
     case ImageInfoRegistryType:
     {
-      DestroyImageInfo((ImageInfo *) p->blob);
+      p->blob=DestroyImageInfo((ImageInfo *) p->blob);
       break;
     }
     default:
@@ -193,12 +194,12 @@ static void *DestroyRegistryElement(void *registry_info)
 
 MagickExport void DestroyMagickRegistry(void)
 {
-  AcquireSemaphoreInfo(&registry_semaphore);
+  registry_semaphore=AcquireSemaphoreInfo(registry_semaphore);
   if (registry_list != (LinkedListInfo *) NULL)
-    DestroyLinkedList(registry_list,DestroyRegistryElement);
+    registry_list=DestroyLinkedList(registry_list,DestroyRegistryElement);
   registry_list=(LinkedListInfo *) NULL;
-  RelinquishSemaphoreInfo(&registry_semaphore);
-  DestroySemaphoreInfo(&registry_semaphore);
+  RelinquishSemaphoreInfo(registry_semaphore);
+  registry_semaphore=DestroySemaphoreInfo(registry_semaphore);
 }
 
 /*
@@ -239,13 +240,16 @@ MagickExport Image *GetImageFromMagickRegistry(const char *name,long *id,
   register const RegistryInfo
     *p;
 
-  if (registry_list == (LinkedListInfo *) NULL)
-    return((Image *) NULL);
-  if (IsLinkedListEmpty(registry_list) == True)
-    return((Image *) NULL);
+  if ((registry_list == (LinkedListInfo *) NULL) ||
+      (IsLinkedListEmpty(registry_list) == MagickTrue))
+    {
+      (void) ThrowMagickException(exception,GetMagickModule(),RegistryError,
+        "UnableToLocateRegistryImage",name);
+      return((Image *) NULL);
+    }
   *id=(-1);
   image=(Image *) NULL;
-  AcquireSemaphoreInfo(&registry_semaphore);
+  registry_semaphore=AcquireSemaphoreInfo(registry_semaphore);
   ResetLinkedListIterator(registry_list);
   p=(const RegistryInfo *) GetNextElementInLinkedList(registry_list);
   while (p != (const RegistryInfo *) NULL)
@@ -259,10 +263,10 @@ MagickExport Image *GetImageFromMagickRegistry(const char *name,long *id,
       }
     p=(const RegistryInfo *) GetNextElementInLinkedList(registry_list);
   }
-  RelinquishSemaphoreInfo(&registry_semaphore);
+  RelinquishSemaphoreInfo(registry_semaphore);
   if (image == (Image *) NULL)
     (void) ThrowMagickException(exception,GetMagickModule(),RegistryError,
-      "UnableToLocateImage",name);
+      "UnableToLocateRegistryImage",name);
   return(image);
 }
 
@@ -306,14 +310,24 @@ MagickExport void *GetMagickRegistry(const long id,RegistryType *type,
   void
     *blob;
 
+  assert(type != (RegistryType *) NULL);
+  assert(length != (size_t *) NULL);
+  assert(exception != (ExceptionInfo *) NULL);
+  if ((registry_list == (LinkedListInfo *) NULL) ||
+      (IsLinkedListEmpty(registry_list) == MagickTrue))
+    {
+      char
+        reason[MaxTextExtent];
+
+      (void) FormatMagickString(reason,MaxTextExtent,"id=%ld",id);
+      (void) ThrowMagickException(exception,GetMagickModule(),RegistryError,
+        "UnableToGetRegistryID",reason);
+      return((Image *) NULL);
+    }
   blob=(void *) NULL;
   *type=UndefinedRegistryType;
   *length=0;
-  if (registry_list == (LinkedListInfo *) NULL)
-    return((Image *) NULL);
-  if (IsLinkedListEmpty(registry_list) == True)
-    return((Image *) NULL);
-  AcquireSemaphoreInfo(&registry_semaphore);
+  registry_semaphore=AcquireSemaphoreInfo(registry_semaphore);
   ResetLinkedListIterator(registry_list);
   p=(const RegistryInfo *) GetNextElementInLinkedList(registry_list);
   while (p != (const RegistryInfo *) NULL)
@@ -360,7 +374,7 @@ MagickExport void *GetMagickRegistry(const long id,RegistryType *type,
       *type=p->type;
       *length=p->length;
     }
-  RelinquishSemaphoreInfo(&registry_semaphore);
+  RelinquishSemaphoreInfo(registry_semaphore);
   if (blob == (void *) NULL)
     {
       char
@@ -408,12 +422,12 @@ MagickExport void *GetMagickRegistry(const long id,RegistryType *type,
 MagickExport long SetMagickRegistry(const RegistryType type,const void *blob,
   const size_t length,ExceptionInfo *exception)
 {
+  MagickBooleanType
+    status;
+
   RegistryInfo
     *registry_info;
 
-  unsigned int
-    status;
-
   void
     *clone_blob;
 
@@ -473,16 +487,16 @@ MagickExport long SetMagickRegistry(const RegistryType type,const void *blob,
       (void) CopyMagickMemory(clone_blob,blob,length);
     }
   }
-  registry_info=(RegistryInfo *) AcquireMagickMemory(sizeof(RegistryInfo));
+  registry_info=(RegistryInfo *) AcquireMagickMemory(sizeof(*registry_info));
   if (registry_info == (RegistryInfo *) NULL)
     ThrowMagickFatalException(ResourceLimitFatalError,
       "UnableToAllocateRegistryInfo",strerror(errno));
-  (void) ResetMagickMemory(registry_info,0,sizeof(RegistryInfo));
+  (void) ResetMagickMemory(registry_info,0,sizeof(*registry_info));
   registry_info->type=type;
   registry_info->blob=clone_blob;
   registry_info->length=length;
   registry_info->signature=MagickSignature;
-  AcquireSemaphoreInfo(&registry_semaphore);
+  registry_semaphore=AcquireSemaphoreInfo(registry_semaphore);
   registry_info->id=id++;
   if (registry_list == (LinkedListInfo *) NULL)
     {
@@ -492,9 +506,9 @@ MagickExport long SetMagickRegistry(const RegistryType type,const void *blob,
           "UnableToAllocateRegistryInfo",strerror(errno));
     }
   status=AppendElementToLinkedList(registry_list,registry_info);
-  if (status == False)
+  if (status == MagickFalse)
     (void) ThrowMagickException(exception,GetMagickModule(),ResourceLimitError,
       "MemoryAllocationFailed",strerror(errno));
-  RelinquishSemaphoreInfo(&registry_semaphore);
+  RelinquishSemaphoreInfo(registry_semaphore);
   return(registry_info->id);
 }
